#define _VFUNC_GEAR_C_
/*
   --------------------------------------------------------------
   File vfunc_gear.C 
   of ADOL-C version 1.8.0                        as of Dec/01/98
   --------------------------------------------------------------

   Example: function module containing the machine tool example
            of gearing

   Each << function module >> contains:
          
     (1) const char* const controlFileName 
     (2) int indepDim; 
     (3) int depDim; 

     (4) void initProblemParameters( void )
     (5) void initIndependents( double* indeps )
     (6) void originalVectorFunction( double* indeps, double* deps )
     (7) void tapingVectorFunction( int tag, double* indeps, double* deps )   

   Last changes: 
     981201 olvo new headers
     980805 olvo first version

   --------------------------------------------------------------
*/


/****************************************************************************/
/*                                                                 INCLUDES */
#include "adouble.h"

#include <math.h>
#include <time.h>
#include <stdlib.h>


/****************************************************************************/
/*                                                         GLOBAL VARIABLES */

/*--------------------------------------------------------------------------*/
/*                                                        Control file name */
const char* controlFileName = "gearexam.ctrl";

/*--------------------------------------------------------------------------*/
/*                                                               Dimensions */
int indepDim;
int depDim;

/*--------------------------------------------------------------------------*/
/*                                       Other problem dependent parameters */
static unsigned short int dx[3]; /* variable needed by erand48(.) */


/****************************************************************************/
/*                                                  INIT PROBLEM PARAMETERS */
void initProblemParameters( void )
{ fprintf(stdout,"GEAREXAM (ADOL-C Example)\n\n");

  /* number of indeps & deps */ 
  indepDim = 3;
  depDim   = 3;

  /* Init erand48(); */
  struct tm s;
  time_t t;
  time(&t); 
  s=*localtime(&t);
  srand(s.tm_sec*s.tm_min); 
  dx[0]=rand();
  dx[1]=rand();
  dx[2]=rand();
}


/****************************************************************************/
/*                                                        INITIALIZE INDEPs */
void initIndependents( double* indeps )
{ for (int i=0; i<indepDim; i++)
     indeps[i] = erand48(dx);
}


/****************************************************************************/
/*                                                 ORIGINAL SCALAR FUNCTION */

/*--------------------------------------------------------------------------*/
/*                                                       The model function */
#define Pi 3.141592654

// Parameter fuer Zahnradherstellung

int konvex=     1;          // konvexe oder konkave Flanke

int zz=         13;         // Zaehnezahl
double delta=   16.46;      // Kegelwinkel
double st=      0.00001;    // Verschiebung Teilkegelspitze

double xmw=     79.33279;   // MK-x
double ymw=     -74.05;     // MK-y
double zmw=     159.49741;  // MK-z
double m=       0.0;        // Erzeugungs-Achsversatz
double zwr=     0.0;        // Reitstockeinstellung
double theta_w= -58.2253;   // Waelztrommelwinkel

double xmk=     17.49874;   // Messerversatz
int zm=         5;          // MK-Gangzahl
double ymk=     80.0;       // MK-Versatz

double rho=     101.64155;  // Spitzenradius, Flugkreisr.
double r=       2.1;        // Kopfradius
double rs=      2594.425;   // Sphaerikradius
double ys=      876.147;    // Sphaerik-Mitte-Y
double zs=      -2442.015;  // Sphaerik-Mitte-Z  


// Allokation und Belegung einer Rotationsmatrix um x-Achse

double** Rot1( double phi) {
  int i;

  double** A;
  A= new double*[3];
  for(i=0; i<3; i++)
    A[i]=new double[3];

  A[0][0]=1.0;  A[0][1]=0.0;       A[0][2]=0.0;
  A[1][0]=0.0;  A[1][1]=cos(phi);  A[1][2]=-sin(phi);
  A[2][0]=0.0;  A[2][1]=sin(phi);  A[2][2]=cos(phi);

  return(A);
}

adouble** Rot1(adouble phi) {
  int i;

  adouble** A;
  A= new adouble*[3];
  for(i=0; i<3; i++)
    A[i]=new adouble[3];

  A[0][0]=1.0;  A[0][1]=0.0;       A[0][2]=0.0;
  A[1][0]=0.0;  A[1][1]=cos(phi);  A[1][2]=-sin(phi);
  A[2][0]=0.0;  A[2][1]=sin(phi);  A[2][2]=cos(phi);

  return(A);
}


// Allokation und Belegung einer Rotationsmatrix um y-Achse

double** Rot2( double phi) {
  int i;

  double** A;
  A= new double*[3];
  for(i=0; i<3; i++)
    A[i]=new double[3];

  A[0][0]=cos(phi);   A[0][1]=0.0;  A[0][2]=sin(phi);
  A[1][0]=0.0;        A[1][1]=1.0;  A[1][2]=0.0;
  A[2][0]=-sin(phi);  A[2][1]=0.0;  A[2][2]=cos(phi);

  return(A);
}

adouble** Rot2(adouble phi) {
  int i;

  adouble** A;
  A= new adouble*[3];
  for(i=0; i<3; i++)
    A[i]=new adouble[3];

  A[0][0]=cos(phi);   A[0][1]=0.0;  A[0][2]=sin(phi);
  A[1][0]=0.0;        A[1][1]=1.0;  A[1][2]=0.0;
  A[2][0]=-sin(phi);  A[2][1]=0.0;  A[2][2]=cos(phi);

  return(A);
}


// Allokation und Belegung einer Rotationsmatrix um z-Achse 

double** Rot3( double phi) {
  int i;

  double** A;
  A= new double*[3];
  for(i=0; i<3; i++)
    A[i]=new double[3];

  A[0][0]=cos(phi);  A[0][1]=-sin(phi);  A[0][2]=0.0;
  A[1][0]=sin(phi);  A[1][1]=cos(phi);   A[1][2]=0.0;
  A[2][0]=0.0;       A[2][1]=0.0;        A[2][2]=1.0;

  return(A);
}

adouble** Rot3(adouble phi) {
  int i;

  adouble** A;
  A= new adouble*[3];
  for(i=0; i<3; i++)
    A[i]=new adouble[3];

  A[0][0]=cos(phi);  A[0][1]=-sin(phi);  A[0][2]=0.0;
  A[1][0]=sin(phi);  A[1][1]=cos(phi);   A[1][2]=0.0;
  A[2][0]=0.0;       A[2][1]=0.0;        A[2][2]=1.0;

  return(A);
}



void def_messer(adouble *z, adouble *messer) {

  double u0, uOri, phi0;
  adouble h;

  phi0= asin((r+ys)/(r+rs));  
  if (konvex==1) {
    u0=rs*phi0;
    uOri=1.0;
  }
  else {
    u0=rs*(phi0-Pi);
    uOri=-1.0;
  };

  h= (z[0]-u0)/(uOri*rs);
  messer[0]=zs+rs*cos(h);
  messer[1]=0.0;
  messer[2]=-ys-rs*sin(h);
}

void def_messer(double *z, double *messer) {

  double u0, uOri, phi0;
  double h;

  phi0= asin((r+ys)/(r+rs));
  if (konvex==1) {
    u0=rs*phi0;
    uOri=1.0;
  }
  else {
    u0=rs*(phi0-Pi);
    uOri=-1.0;
  };

  h= (z[0]-u0)/(uOri*rs);
  messer[0]=zs+rs*cos(h);
  messer[1]=0.0;
  messer[2]=-ys-rs*sin(h);
}

void gearFunction(double* pz, double* pf) 
{ int i,j;

  double ah;
  double* z;
  z= new double[3];
  for(i=0; i<3; i++)
    z[i] = pz[i];

  double* messer;
  messer= new double[3];
  def_messer(z, messer);

  double* mess_new;
  mess_new= new double[3];


  // Position der Schneide am Messerkopf
  messer[0]+=rho;
  messer[1]-=xmk;

  // Messerkopfrotation mit Parameter v
  double **D3_v=Rot3(z[1]);
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D3_v[i][j]*messer[j];
  } 

  // Lage des Messerkopfs auf der Wiege
  mess_new[2]-=ymk;

  // Eindrehen in Orientierung der Wiege
  ah=mess_new[0];
  mess_new[0]=mess_new[1];
  mess_new[1]=ah;
  mess_new[2]=-mess_new[2];
 
  // Verschiebung
  mess_new[0]-=xmw;
  mess_new[1]+=zmw;
  mess_new[2]+=ymw;

  // Wiegenwinkel thetaW, entspricht dem wert t=0
  double **D3_thetaw=Rot3(theta_w);
  for(i=0; i<3; i++) {
    messer[i]=0.0;
    for(j=0; j<3; j++)
      messer[i]+=D3_thetaw[i][j]*mess_new[j];
  }

  // Wiegenbewegung mit Parameter t
  double **D3_t=Rot3(z[2]);
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D3_t[i][j]*messer[j];
  }

  // Achsversatz
  mess_new[0]+=m;

  // Eindrehen in Orientierung des Werkrades
  mess_new[0]=-mess_new[0];
  ah=mess_new[1];
  mess_new[1]=-mess_new[2];
  mess_new[2]=-ah;

  // Teilkegeloeffnungswinkel delta, y-Achsen entgegengesetzt
  double **D1_delta=Rot1(delta);
  for(i=0; i<3; i++) {
    messer[i]=0.0;
    for(j=0; j<3; j++)
      messer[i]+=D1_delta[i][j]*mess_new[j];
  }

  // neue Verschiebung der Werkradachse
  messer[2]+=zwr+st;

  // gekoppelte Werkraddrehung in Abhaengigkeit von t und v
  double **D2_wt_cv=Rot2( (1/sin(delta))*z[2] + (double(zm)/zz)*z[1] );
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D2_wt_cv[i][j]*messer[j];
  }

  for(i=0; i<3; i++)
    pf[i] = mess_new[i];
  //printf(" %f %f %f \n",pf[0],pf[1],pf[2]);

  for(i=0; i<3; i++)
    delete[] D2_wt_cv[i];
  delete[] D2_wt_cv;
  for(i=0; i<3; i++)
    delete[] D1_delta[i];
  delete[] D1_delta;
  for(i=0; i<3; i++)
    delete[] D3_t[i];
  delete[] D3_t;
  for(i=0; i<3; i++)
    delete[] D3_thetaw[i];
  delete[] D3_thetaw;
  for(i=0; i<3; i++)
    delete[] D3_v[i];
  delete[] D3_v;
  delete[] mess_new;
  delete[] messer;
  delete[] z;
}

/*--------------------------------------------------------------------------*/
/*                                                   The interface function */
void originalVectorFunction( double* indeps, double* deps )
{ gearFunction(indeps,deps);
}


/****************************************************************************/
/*                                                   TAPING SCALAR FUNCTION */

/*--------------------------------------------------------------------------*/
/*                                                       The model function */
void activeGearFunction(adouble* z, adouble* mess_new) 
{ int i,j;
  adouble ah;

  adouble* messer = new adouble[3];
  def_messer(z, messer);

  // Position der Schneide am Messerkopf
  messer[0]+=rho;
  messer[1]-=xmk;

  // Messerkopfrotation mit Parameter v
  adouble **D3_v=Rot3(z[1]);
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D3_v[i][j]*messer[j];
  }  

  // Lage des Messerkopfs auf der Wiege
  mess_new[2]-=ymk;

  // Eindrehen in Orientierung der Wiege
  ah=mess_new[0];
  mess_new[0]=mess_new[1];
  mess_new[1]=ah;
  mess_new[2]=-mess_new[2];
  
  // Verschiebung
  mess_new[0]-=xmw;
  mess_new[1]+=zmw;
  mess_new[2]+=ymw;

  // Wiegenwinkel thetaW, entspricht dem wert t=0
  double **D3_thetaw=Rot3(theta_w);
  for(i=0; i<3; i++) {
    messer[i]=0.0;
    for(j=0; j<3; j++)
      messer[i]+=D3_thetaw[i][j]*mess_new[j];
  }

  // Wiegenbewegung mit Parameter t
  adouble **D3_t=Rot3(z[2]);
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D3_t[i][j]*messer[j];
  }
  // Achsversatz
  mess_new[0]+=m;

  // Eindrehen in Orientierung des Werkrades
  mess_new[0]=-mess_new[0];
  ah=mess_new[1];
  mess_new[1]=-mess_new[2];
  mess_new[2]=-ah;

  // Teilkegeloeffnungswinkel delta, y-Achsen entgegengesetzt
  double **D1_delta=Rot1(delta);
  for(i=0; i<3; i++) {
    messer[i]=0.0;
    for(j=0; j<3; j++)
      messer[i]+=D1_delta[i][j]*mess_new[j];
  }

  // neue Verschiebung der Werkradachse
  messer[2]+=zwr+st;

  // gekoppelte Werkraddrehung in Abhaengigkeit von t und v
  adouble **D2_wt_cv=Rot2( (1/sin(delta))*z[2] + (double(zm)/zz)*z[1] );
  for(i=0; i<3; i++) {
    mess_new[i]=0.0;
    for(j=0; j<3; j++)
      mess_new[i]+=D2_wt_cv[i][j]*messer[j];
  }

  for(i=0; i<3; i++)
    delete[] D2_wt_cv[i];
  delete[] D2_wt_cv;
  for(i=0; i<3; i++)
    delete[] D1_delta[i];
  delete[] D1_delta;
  for(i=0; i<3; i++)
    delete[] D3_t[i];
  delete[] D3_t;
  for(i=0; i<3; i++)
    delete[] D3_thetaw[i];
  delete[] D3_thetaw;
  for(i=0; i<3; i++)
    delete[] D3_v[i];
  delete[] D3_v;
  delete[] messer;
}


/*--------------------------------------------------------------------------*/
/*                                                   The interface function */
void tapingVectorFunction( int tag, double* indeps, double* deps )
{ int i;
  trace_on(tag);
  adouble* activeIndeps = new adouble[indepDim];
  adouble* activeDeps   = new adouble[depDim];
  adouble* aIP = activeIndeps;
  double*  iP  = indeps; 
  for (i=0; i<indepDim; i++)
     *aIP++ <<= *iP++; 
  activeGearFunction(activeIndeps,activeDeps);
  aIP = activeDeps;
  iP  = deps; 
  for (i=0; i<depDim; i++)
    *aIP++ >>= *iP++; 
  trace_off();
}

#undef _VFUNC_GEAR_C_





